Attribute VB_Name = "OGLUtils"
Option Explicit

' a couple of declares to work around some deficiencies of the type library
Private Declare Function EnumDisplaySettings Lib "user32" Alias "EnumDisplaySettingsA" (ByVal lpszDeviceName As Long, ByVal iModeNum As Long, lpDevMode As Any) As Boolean
Private Declare Function ChangeDisplaySettings Lib "user32" Alias "ChangeDisplaySettingsA" (lpDevMode As Any, ByVal dwflags As Long) As Long
Private Declare Function CreateIC Lib "gdi32" Alias "CreateICA" (ByVal lpDriverName As String, ByVal lpDeviceName As String, ByVal lpOutput As String, ByVal lpInitData As Long) As Long

Private Const CCDEVICENAME = 32
Private Const CCFORMNAME = 32
Private Const DM_BITSPERPEL = &H40000
Private Const DM_PELSWIDTH = &H80000
Private Const DM_PELSHEIGHT = &H100000

Private Type DEVMODE
    dmDeviceName        As String * CCDEVICENAME
    dmSpecVersion       As Integer
    dmDriverVersion     As Integer
    dmSize              As Integer
    dmDriverExtra       As Integer
    dmFields            As Long
    dmOrientation       As Integer
    dmPaperSize         As Integer
    dmPaperLength       As Integer
    dmPaperWidth        As Integer
    dmScale             As Integer
    dmCopies            As Integer
    dmDefaultSource     As Integer
    dmPrintQuality      As Integer
    dmColor             As Integer
    dmDuplex            As Integer
    dmYResolution       As Integer
    dmTTOption          As Integer
    dmCollate           As Integer
    dmFormName          As String * CCFORMNAME
    dmUnusedPadding     As Integer
    dmBitsPerPel        As Integer
    dmPelsWidth         As Long
    dmPelsHeight        As Long
    dmDisplayFlags      As Long
    dmDisplayFrequency  As Long
End Type

Public Keys(255) As Boolean             ' used to keep track of key_downs

Private hrc As Long
Private fullscreen As Boolean

Private OldWidth As Long
Private OldHeight As Long
Private OldBits As Long
Private OldVertRefresh As Long
Public xrot As Single                                  ' X Rotation ( NEW )
Public yrot As Single                                  ' Y Rotation ( NEW )
Public zrot As Single                                  ' Z Rotation ( NEW )

Public texture(1) As Long                             ' Storage For One Texture ( NEW )
Private mPointerCount As Integer

Private cGLext_ARB As New ClsGLext_ARB 'GL Extensions Class

'Multitexture Constants
Private Const GL_TEXTURE0_ARB = 33984 ' &H84C0
Private Const GL_TEXTURE1_ARB = 33985 '&H84C1
Private Const GL_COMBINE_ARB = 34160 '&H8570
Private Const GL_RGB_SCALE_ARB = 34163 ' &H8573

'Private Const GL_TEXTURE0_ARB = &H84C0
'Private Const GL_TEXTURE1_ARB = &H84C1
'Private Const GL_COMBINE_ARB = &H8570
'Private Const GL_RGB_SCALE_ARB = &H8573


Private Function LoadBMP(ByVal Filename As String, ByRef texture() As Byte, ByRef height As Long, ByRef width As Long) As Boolean
    Form1.Picture1.Picture = LoadPicture(Filename)  ' note the SacleMode of the picture control is set to 3 - Pixel
    CreateMapImage Form1.Picture1, texture(), height, width ' Create a texture map array from the picture
    LoadBMP = True
End Function

Private Sub CreateMapImage(pict As PictureBox, ByRef TextureImg() As Byte, ByRef height As Long, ByRef width As Long)
    ' Create the array as needed for the image.
    pict.ScaleMode = 3                  ' Pixels
    height = pict.ScaleHeight
    width = pict.ScaleWidth
    
    ReDim TextureImg(2, height - 1, width - 1) ' size our texture array
    
    Dim x As Long
    Dim y As Long
    Dim c As Long
    
    Dim yloc As Long
    For x = 0 To width - 1                      ' loop through every pixel in the image
        For y = 0 To height - 1
            c = pict.Point(x, y)                ' Returns in long format.
            yloc = height - y - 1               ' work out where we are in the Y
            TextureImg(0, x, yloc) = c And &HFF     ' get the lower 8 bits Red
            TextureImg(1, x, yloc) = (c \ 256) And &HFF   ' Get the middle 8 bits Green
            TextureImg(2, x, yloc) = (c \ 65536) And &HFF   'Get the top 8 bits Blue
        Next y
    Next x
    
End Sub


Private Function LoadGLTextures() As Boolean
' Load Bitmaps And Convert To Textures
    Dim Status As Boolean
    Dim h As Long
    Dim w As Long
    Dim TextureImage() As Byte
    Status = False                         ' Status Indicator

    If LoadBMP(App.Path & "\Data\image1.bmp", TextureImage(), h, w) Then
        ' Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
        Status = True                          ' Set The Status To TRUE

        glGenTextures 1, texture(0)                 ' Create The Texture

        ' Typical Texture Generation Using Data From The Bitmap
        glBindTexture GL_TEXTURE_2D, texture(0)

        ' Generate The Texture
        glTexImage2D GL_TEXTURE_2D, 0, 3, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage(0, 0, 0)
        glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR     ' Linear Filtering
        glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR     ' Linear Filtering
        'Call glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE)


    End If

    Erase TextureImage   ' Free the texture image memory
    
    If LoadBMP(App.Path & "\Data\Lightmap.bmp", TextureImage(), h, w) Then
        ' Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
        Status = True                          ' Set The Status To TRUE

        glGenTextures 1, texture(1)                 ' Create The Texture

        ' Typical Texture Generation Using Data From The Bitmap
        glBindTexture GL_TEXTURE_2D, texture(1)

        ' Generate The Texture
        glTexImage2D GL_TEXTURE_2D, 0, 3, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage(0, 0, 0)
        glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR     ' Linear Filtering
        glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR     ' Linear Filtering
        'Call glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE)

    End If

    Erase TextureImage   ' Free the texture image memory
    
    
    LoadGLTextures = Status
End Function


Public Sub ReSizeGLScene(ByVal width As Integer, ByVal height As Integer)
' Resize And Initialize The GL Window
    If height = 0 Then              ' Prevent A Divide By Zero By
        height = 1                  ' Making Height Equal One
    End If
    glViewport 0, 0, width, height  ' Reset The Current Viewport
    glMatrixMode GL_PROJECTION       ' Select The Projection Matrix
    glLoadIdentity                  ' Reset The Projection Matrix

    ' Calculate The Aspect Ratio Of The Window
    gluPerspective 45#, width / height, 0.1, 100#

    glMatrixMode GL_MODELVIEW        ' Select The Modelview Matrix
    glLoadIdentity                  ' Reset The Modelview Matrix
End Sub

Public Function InitGL() As Boolean
' All Setup For OpenGL Goes Here
    If Not LoadGLTextures Then          ' Jump To Texture Loading Routine ( NEW )
        InitGL = False                  ' If Texture Didn't Load Return FALSE ( NEW )
        Exit Function
    End If

    glEnable GL_TEXTURE_2D               ' Enable Texture Mapping ( NEW )
    glShadeModel GL_SMOOTH               ' Enables Smooth Shading

    glClearColor 1#, 0#, 0#, 0#         ' Black Background

    glClearDepth 1#                     ' Depth Buffer Setup
    glEnable GL_DEPTH_TEST               ' Enables Depth Testing
    glDepthFunc GL_LEQUAL                ' The Type Of Depth Test To Do

    'glHint htPerspectiveCorrectionHint, hmNicest    ' Really Nice Perspective Calculations

 'glActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC)wglGetProcAddress("glActiveTextureARB");
 'glMultiTexCoord2fARB = (PFNGLMULTITEXCOORD2FARBPROC)wglGetProcAddress("glMultiTexCoord2fARB");
    
    'Debug.Print cGLext_ARB.glActiveTextureARB", vbNull) ', VarPtr("PFNGLACTIVETEXTUREARBPROC"))
    'Debug.Print cGLext_ARB.glMultiTexCoord2fARB", vbNull, vbNull, vbNull) 'VarPtr("PFNGLMULTITEXCOORD2FARBPROC"))

    InitGL = True                       ' Initialization Went OK
End Function


Public Sub KillGLWindow()
' Properly Kill The Window

    If hrc Then                                     ' Do We Have A Rendering Context?
        If wglMakeCurrent(0, 0) = 0 Then             ' Are We Able To Release The DC And RC Contexts?
            MsgBox "Release Of DC And RC Failed.", vbInformation, "SHUTDOWN ERROR"
        End If

        If wglDeleteContext(hrc) = 0 Then           ' Are We Able To Delete The RC?
            MsgBox "Release Rendering Context Failed.", vbInformation, "SHUTDOWN ERROR"
        End If
        hrc = 0                                     ' Set RC To NULL
    End If

    ' Note
    ' The form owns the device context (hDC) window handle (hWnd) and class (RTThundermain)
    ' so we do not have to do all the extra work

End Sub


Private Function FindDEVMODE(ByVal width As Integer, ByVal height As Integer, ByVal Bits As Integer, Optional ByVal VertRefresh As Long = -1) As DEVMODE
    ' locate a DEVMOVE that matches the passed parameters
    Dim ret As Boolean
    Dim i As Long
    Dim dm As DEVMODE
    i = 0
    Do  ' enumerate the display settings until we find the one we want
        ret = EnumDisplaySettings(0&, i, dm)
        If dm.dmPelsWidth = width And _
            dm.dmPelsHeight = height And _
            dm.dmBitsPerPel = Bits And _
            ((dm.dmDisplayFrequency = VertRefresh) Or (VertRefresh = -1)) Then Exit Do ' exit when we have a match
        i = i + 1
    Loop Until (ret = False)
    FindDEVMODE = dm
End Function


Public Function CreateGLWindow(frm As Form, width As Integer, height As Integer, Bits As Integer, fullscreenflag As Boolean) As Boolean
    Dim PixelFormat As Long                       ' Holds The Results After Searching For A Match
    Dim pfd As PIXELFORMATDESCRIPTOR                ' pfd Tells Windows How We Want Things To Be


    fullscreen = fullscreenflag                     ' Set The Global Fullscreen Flag

    pfd.cColorBits = Bits
    pfd.cDepthBits = 16
    pfd.dwflags = PFD_DRAW_TO_WINDOW Or PFD_SUPPORT_OPENGL Or PFD_DOUBLEBUFFER Or PFD_TYPE_RGBA
    pfd.iLayerType = PFD_MAIN_PLANE
    pfd.iPixelType = PFD_TYPE_RGBA
    pfd.nSize = Len(pfd)
    pfd.nVersion = 1
    
    PixelFormat = ChoosePixelFormat(frm.hDC, pfd)
    If PixelFormat = 0 Then                     ' Did Windows Find A Matching Pixel Format?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Find A Suitable PixelFormat.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If

    If SetPixelFormat(frm.hDC, PixelFormat, pfd) = 0 Then ' Are We Able To Set The Pixel Format?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Set The PixelFormat.", vbExclamation, "ERROR"
        CreateGLWindow = False                           ' Return FALSE
    End If
    
    hrc = wglCreateContext(frm.hDC)
    If (hrc = 0) Then                           ' Are We Able To Get A Rendering Context?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Create A GL Rendering Context.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If

    If wglMakeCurrent(frm.hDC, hrc) = 0 Then    ' Try To Activate The Rendering Context
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Activate The GL Rendering Context.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If
    frm.Show                                    ' Show The Window
    'SetForegroundWindow frm.hWnd                ' Slightly Higher Priority
    frm.SetFocus                                ' Sets Keyboard Focus To The Window
    ReSizeGLScene frm.ScaleWidth, frm.ScaleHeight ' Set Up Our Perspective GL Screen

    If Not InitGL() Then                        ' Initialize Our Newly Created GL Window
        KillGLWindow                            ' Reset The Display
        MsgBox "Initialization Failed.", vbExclamation, "ERROR"
        CreateGLWindow = False                   ' Return FALSE
    End If

    CreateGLWindow = True                       ' Success

End Function

Public Function DrawGLScene() As Boolean

' Here's Where We Do All The Drawing
    glClear GL_COLOR_BUFFER_BIT Or GL_DEPTH_BUFFER_BIT            ' Clear Screen And Depth Buffer
    glLoadIdentity                            ' Reset The Current Matrix
    glTranslatef 0#, 0#, -5#                          ' Move Into The Screen 5 Units
    'glRotatef xrot, 1#, 0#, 0#                        ' Rotate On The X Axis
    'glRotatef yrot, 0#, 1#, 0#                        ' Rotate On The Y Axis
    'glRotatef zrot, 0#, 0#, 1#                        ' Rotate On The Z Axis

   ' Call glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_ARB)
   ' Call glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_ARB, 2)


    Call cGLext_ARB.glActiveTextureARB(GL_TEXTURE0_ARB)
    Call glEnable(GL_TEXTURE_2D)
    Call glBindTexture(GL_TEXTURE_2D, texture(0))
    Dim x As Long, y As Long
    
    Call cGLext_ARB.glActiveTextureARB(GL_TEXTURE1_ARB)
    Call glEnable(GL_TEXTURE_2D)
    Call glBindTexture(GL_TEXTURE_2D, texture(1))
    
    
    glBegin (GL_QUADS)
        '// top left vertex
        x = 0: y = 0
        Call cGLext_ARB.glMultiTexCoord2iARB(GL_TEXTURE0_ARB, x, y)
        Call cGLext_ARB.glMultiTexCoord2iARB(GL_TEXTURE1_ARB, x, y)
        Call glVertex3f(-1, 1, 0)
        '// bottom left vertex
        x = 0: y = 1
        Call cGLext_ARB.glMultiTexCoord2iARB(GL_TEXTURE0_ARB, x, y)
        Call cGLext_ARB.glMultiTexCoord2iARB(GL_TEXTURE1_ARB, x, y)
        Call glVertex3f(-1, -1, 0)
        x = 1: y = 1
        '// bottom right vertex)
        Call cGLext_ARB.glMultiTexCoord2iARB(GL_TEXTURE0_ARB, x, y)
        Call cGLext_ARB.glMultiTexCoord2iARB(GL_TEXTURE1_ARB, x, y)
        Call glVertex3f(1, -1, 0)
        '// top right vertex
        x = 1: y = 0
        Call cGLext_ARB.glMultiTexCoord2iARB(GL_TEXTURE0_ARB, x, y)
        Call cGLext_ARB.glMultiTexCoord2iARB(GL_TEXTURE1_ARB, x, y)
        Call glVertex3f(1, 1, 0)
    glEnd


    Call cGLext_ARB.glActiveTextureARB(GL_TEXTURE1_ARB)
    Call glDisable(GL_TEXTURE_2D) 'Disable the 2nd texture

    Call cGLext_ARB.glActiveTextureARB(GL_TEXTURE0_ARB)


'    glBindTexture GL_TEXTURE_2D, Texture(0)           ' Select Our Texture
'
'    glBegin GL_QUADS
'        ' Front Face
'        glTexCoord2f 0#, 0#: glVertex3f -1#, -1#, 1#             ' Bottom Left Of The Texture and Quad
'        glTexCoord2f 1#, 0#: glVertex3f 1#, -1#, 1#              ' Bottom Right Of The Texture and Quad
'        glTexCoord2f 1#, 1#: glVertex3f 1#, 1#, 1#               ' Top Right Of The Texture and Quad
'        glTexCoord2f 0#, 1#: glVertex3f -1#, 1#, 1#              ' Top Left Of The Texture and Quad
'        ' Back Face
'        glTexCoord2f 1#, 0#: glVertex3f -1#, -1#, -1#            ' Bottom Right Of The Texture and Quad
'        glTexCoord2f 1#, 1#: glVertex3f -1#, 1#, -1#             ' Top Right Of The Texture and Quad
'        glTexCoord2f 0#, 1#: glVertex3f 1#, 1#, -1#              ' Top Left Of The Texture and Quad
'        glTexCoord2f 0#, 0#: glVertex3f 1#, -1#, -1#             ' Bottom Left Of The Texture and Quad
'        ' Top Face
'        glTexCoord2f 0#, 1#: glVertex3f -1#, 1#, -1#             ' Top Left Of The Texture and Quad
'        glTexCoord2f 0#, 0#: glVertex3f -1#, 1#, 1#              ' Bottom Left Of The Texture and Quad
'        glTexCoord2f 1#, 0#: glVertex3f 1#, 1#, 1#               ' Bottom Right Of The Texture and Quad
'        glTexCoord2f 1#, 1#: glVertex3f 1#, 1#, -1#              ' Top Right Of The Texture and Quad
'        ' Bottom Face
'        glTexCoord2f 1#, 1#: glVertex3f -1#, -1#, -1#            ' Top Right Of The Texture and Quad
'        glTexCoord2f 0#, 1#: glVertex3f 1#, -1#, -1#             ' Top Left Of The Texture and Quad
'        glTexCoord2f 0#, 0#: glVertex3f 1#, -1#, 1#              ' Bottom Left Of The Texture and Quad
'        glTexCoord2f 1#, 0#: glVertex3f -1#, -1#, 1#             ' Bottom Right Of The Texture and Quad
'        ' Right face
'        glTexCoord2f 1#, 0#: glVertex3f 1#, -1#, -1#             ' Bottom Right Of The Texture and Quad
'        glTexCoord2f 1#, 1#: glVertex3f 1#, 1#, -1#              ' Top Right Of The Texture and Quad
'        glTexCoord2f 0#, 1#: glVertex3f 1#, 1#, 1#               ' Top Left Of The Texture and Quad
'        glTexCoord2f 0#, 0#: glVertex3f 1#, -1#, 1#              ' Bottom Left Of The Texture and Quad
'        ' Left Face
'        glTexCoord2f 0#, 0#: glVertex3f -1#, -1#, -1#            ' Bottom Left Of The Texture and Quad
'        glTexCoord2f 1#, 0#: glVertex3f -1#, -1#, 1#             ' Bottom Right Of The Texture and Quad
'        glTexCoord2f 1#, 1#: glVertex3f -1#, 1#, 1#              ' Top Right Of The Texture and Quad
'        glTexCoord2f 0#, 1#: glVertex3f -1#, 1#, -1#             ' Top Left Of The Texture and Quad
'    glEnd

    xrot = xrot + 0.3                             ' X Axis Rotation
    yrot = yrot + 0.2                             ' Y Axis Rotation
    zrot = zrot + 0.4                             ' Z Axis Rotation
    DrawGLScene = True                                 ' Keep Going

End Function


Sub Main()
    Dim Done As Boolean
    Dim frm As Form
    Done = False
    ' Ask The User Which Screen Mode They Prefer
    'fullscreen = MsgBox("Would You Like To Run In Fullscreen Mode?", vbYesNo + vbQuestion, "Start FullScreen?") = vbYes
    
    ' Create Our OpenGL Window
    Set frm = New Form1
    If Not CreateGLWindow(frm, 640, 480, 16, fullscreen) Then
        Done = True                             ' Quit If Window Was Not Created
    End If

    Do While Not Done
        ' Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
        If (Not DrawGLScene Or Keys(vbKeyEscape)) Then  ' Updating View Only If Active
            Unload frm                          ' ESC or DrawGLScene Signalled A Quit
        Else                                    ' Not Time To Quit, Update Screen
            SwapBuffers (frm.hDC)               ' Swap Buffers (Double Buffering)
            DoEvents
        End If

        If Keys(vbKeyF1) Then                   ' Is F1 Being Pressed?
            Keys(vbKeyF1) = False               ' If So Make Key FALSE
            Unload frm                          ' Kill Our Current Window
            Set frm = New Form1                 ' create a new one
            fullscreen = Not fullscreen         ' Toggle Fullscreen / Windowed Mode
            ' Recreate Our OpenGL Window
            If Not CreateGLWindow(frm, 640, 480, 16, fullscreen) Then
                Unload frm                      ' Quit If Window Was Not Created
            End If
        End If
        Done = frm.Visible = False              ' if the form is not visible then we are done
    Loop
    ' Shutdown
    Set frm = Nothing
    End
End Sub

